<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * API des phpNSTA für externe Befehle
 *
 * Über die API des phpNSTA kann dieser beendet werden.
 * Der Hauptprozess des phpNSTA begibt, nachdem alle Kindprozesse erstellt in die Endlosschleife API::listener()
 * Von dieser Schleife aus, prüft der phpNSTA ob eventuell ein Kindprozesse gestorben ist und neu erstellt werden muss
 * oder ob sich der phpNSTA beenden soll.
 * @property string $api_key                        Passwort, welches in den API-Kommandos enthalten sein muss
 * @property string $api_cmd                        Pfad + Dateiname der API CMD-Datei
 * @property int    $interval                       Intervall in Sekunden, wie oft auf externe Kommandos geprüft werden soll
 * @property bool   $use_tsync                      Ob der phpNSTA die Uhrzeit auf die SAT-Systeme übertragen soll
 * @property bool   $nagios4x_compatibility         Ob Kompatibilität für Nagios 4 aktiviert ist
 * @property string $nagiostats                     Pfad + Dateiname zur Binary von nagiostats
 * @property int    $nagios_uptime                  Wie lange Nagios laufen muss, bevor der phpNSTA Ergebnisse übergibt
 * @property int    $nagios_uptime_checkinterval    Wie oft die Uptime von Nagios überprüft werden soll in Sekunden
 * @property int    $last_nagios_check              Wann die Uptime von Nagios das letzte mal überprüft wurde, als Timestamp
 * @property array  $known_pids                     Die pids von allen laufenden Kindprozessen
 * @property array  $childs                         Konfiguration alle Kindprozesse
 * @property int    $last_check                     Wann das letzte mal die Supervisor-Verbindungen geprüft wurden als Timestamp
 * @property array  $plannedKill                    Alle pids von Kindprozessen die planmäßig beendet werden sollen
 * @property int    $last_alive_check               Wann zuletzt geprüft wurde, ob alle Kindprozesse noch existieren als Timestamp
 * @property int    $supervisor_interval            Wie oft über supervisor geprüft werden soll, ob alle Verbindungen aufgebaut sind (Sekunden)
 * @property int    $alive_interval                 Wie oft überprüft werden soll, ob alle Kindprozesse existieren (Sekunden)
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @uses Boot - Statische Klasse zum daemon handline
 * @uses TSYNC - Statische Klasse um die Uhrzeit auf die SAT-Systeme zu übertragne
 * @uses SSH - Statische Klasse hält ein array mit allen SAT-Systemen
 * @uses Supervisor, Klasse welche die Kommunikation mit Supervisor übernimmt
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class API{
	
	/**
	 * Function: __construct
	 *
	 * Erstellt das API Object und füllt lokale Variablen
	 *
	 * @since 1.0
	 * @version 1.7.1
	 *
	 */
	function __construct(){
		$this->api_key = Config::val('API', 'api_key');
		$this->api_cmd = Config::val('API', 'cmd_file');
		$this->interval = Config::val('API', 'interval');
		$this->use_tsync = Config::val('TSYNC', 'synchronize_time');
		$this->nagios4x_compatibility = Config::val('NAGIOS', 'nagios4x_compatibility');
		$this->nagiostats = Config::val('NAGIOS', 'nagiostats');
		$this->nagios_uptime = Config::val('NAGIOS', 'nagios_uptime');
		$this->nagios_uptime_checkinterval = Config::val('NAGIOS', 'nagios_uptime_checkinterval');
		$this->last_nagios_check = 0;
		$this->known_pids = array();
		$this->childs = array();
		$this->last_check = time();
		$this->plannedKill = array();
		$this->last_alive_check = time();
		$this->supervisor_interval = Config::val('SUPERVISOR', 'interval');
		$this->alive_interval = Config::val('SSH', 'alive_interval');
		
		if(!file_exists($this->api_cmd)){
			touch($this->api_cmd);
			$this->fixPermissions();

		}
		if(Config::val('SSH', 'use_ssh_tunnel') === true){
			pcntl_signal(SIGCHLD, array($this, 'sigchld_handler'));
			$this->deadChilds = array();
		}
	}
	
	/**
	 * Function: readAPI
	 *
	 * Prüft ob in der CMD Datei neue API-Befehle warten
	 * Wird zum Beispiel von $this->listener() aufgerufen
	 *
	 * @return array Array mit allen Aufbaben aus der API
	 * @since 1.0
	 *
	 */
	function readAPI(){
		clearstatcache();
		if(filesize($this->api_cmd)>0){
			$content = file($this->api_cmd);
			$this->wipe();
			if(is_array($content) && !empty($content)){
				foreach($content as $c){
					$return[] = str_replace("\n", '', $c);
				}
			}
			return $return;
		}
		
		return array();
	}
	
	/**
	 * Function: wipe
	 *
	 * Kürzt die API-CMD Datei des phpNSTA auf eine Länge von 0 und setzt die benötigten Dateirechte
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	function wipe(){
		fclose(fopen($this->api_cmd, 'w+'));
		$this->fixPermissions();
	}
	
	/**
	 * Function: fixPermissions
	 *
	 * Setzt die benötigten Dateirechte, damit Nagios und der Apache Webserver API-Befehle in die CMD Datei des phpNSTA schreiben können
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	function fixPermissions(){
		chown($this->api_cmd, Config::val('NAGIOS', 'nagios_user'));
		chgrp($this->api_cmd, Config::val('NAGIOS', 'apache_group'));
		chmod($this->api_cmd, Config::val('API', 'chmod'));
	}
	
	/**
	 * Function: listener
	 *
	 * Überwacht die API-Schnittstelle und übergibt eventuelle Aufgaben an $this->doTask()
	 *
	 * @return void
	 * @since 1.0
	 * @version 1.7.1
	 *
	 */
	function listener(){
		$use_ssh = Config::val('SSH', 'use_ssh_tunnel');
		if($use_ssh){
			$Supervisor = new Supervisor();
		}
		while(true){
			$a = $this->readAPI();
			if(!empty($a) && is_array($a)){
				foreach($a as $b){
					Logger::log('API Injection! Command: "'.$b.'"', 2);
					$this->doTask($b);
				}
			}
			if(isset($a)){
				unset($a);
			}
			
			if($this->use_tsync === true){
				TSYNC::checkTime();
			}
			
			if($this->nagios4x_compatibility === true && (time() - $this->last_nagios_check) > $this->nagios_uptime_checkinterval){
				exec($this->nagiostats.' -m -d PROGRUNTIMETT', $nagios_runtime);
				if(isset($nagios_runtime[0]) && is_numeric($nagios_runtime[0])){
					// Wenn Nagios nicht läuft gibt nagiostats -m -d PROGRUNTIMETT einen Wert >= 1392215368 zurück -.- (01.01.1970 bis time() ?)
					if($nagios_runtime[0] <= $this->nagios_uptime && $nagios_runtime[0] < (time() - 100)){
						// (time() - 100) sollte eingetlich immer matchen, wenn nagios nicht gestartet ist, wie wir oben gelernt haben...
						Logger::log("Nagios runs for less than ".$this->nagios_uptime." seconds! (Nagios runs for: ".$nagios_runtime[0]." seconds) Starting soon...", 3);
						foreach($this->known_pids as $pid){
							Logger::log('Send my child ['.$pid.'] the signal to stop work (because of Nagios run time)', 2);
							posix_kill($pid, SIGUSR1);
						}
					}else{
						foreach($this->known_pids as $pid){
							Logger::log('Send my child ['.$pid.'] the signal to start/continue work (Nagios runtime looks ok)', 2);
							posix_kill($pid, SIGUSR2);
						}
					}
				}else{
					Logger::log("Can not determine nagios runtime, your phpNSTA may be will never process some data", 3);
				}
				unset($nagios_runtime);
				$this->last_nagios_check = time();
			}
			
			if($use_ssh === true && (time() - $this->last_check) > $this->supervisor_interval){
				Logger::log("Checking connection state via Supervisor", 4);
				$this->last_check = time();
				foreach(SSH::$sat_systems as $sat_system){
					if($Supervisor->checkConnection($sat_system['name']) === false){
						// Prozess muss neugestartet werden, da die VErbindung abgerissen wurde!
						foreach($this->childs as $pid=>$child){
							if(in_array($sat_system['name'], $child['namesAsArray'])){
								Logger::log("Killing my Child cause of lost connection and trying to reconnect", 4);
								// Dieser Child muss neu erstellt werden!
								posix_kill($pid, SIGTERM);
								// Markeirt das dieses Kind planmäßig getötet wird. Sorgt dafür das der sigchild_handler aufgerufen wird und es nicht als segfault oder kill -9 interpertiert wird.
								$this->plannedKill[$pid] = 'killed because of a connection problem';
							}
						}
						sleep(10);
					}
				}
				
			}
			
			//Checking if all childs are alive that should be
			if($use_ssh === true && (time() - $this->last_alive_check) > $this->alive_interval){
				//Checking alive state
				Logger::log('Checking if all childs are alive', 7);
				foreach($this->known_pids as $pid){
					Logger::log('Checking pid: '.$pid, 7);
					//Wenn ein worker vom Parent beendet wird (wegen beenden des phpNSTA oder weil die Verbindung weg ist), wird dies über den sigchild_handler behandelt.
					if(isset($this->plannedKill[$pid])){
						Logger::log('Child ['.$pid.'] is may be not running because of connection problem or you stoped the phpNSTA', 7);
						Logger::log('Child was killed by parrent. API::sigchld_handler() will do the clean up job', 7);
						//continue;
					}
					if(pcntl_waitpid($pid, $status, WNOHANG) > 0){ // -1 child exit correctly
						$dead_child_pid = $pid;
						//This child is dead
						$this->removePid($dead_child_pid);
						Logger::log('Child died! ['.$dead_child_pid.'], detected in line: '.__LINE__, 1);
						Logger::log('Giving the linux kernel a chance to cleanup bind sockets (waiting 1 secs), detected in Line: '.__LINE__, 4);
						sleep(1);
						// Dem Kernel etwas Zeit geben und den Socket zu berreiningen, bevor ein neuer Worker erstellt wird
						foreach($this->childs as $pid=>$child){
							if($pid == $dead_child_pid){
								$this->deadChilds[$dead_child_pid] = $child;
							}
						}
					}
				}
			
				//Restart dead childs
				if(!empty($this->deadChilds)){
					foreach($this->deadChilds as $pid => $child){
						Logger::log('Dead child detected. Trying to restart it', 7);
						$pid_new = pcntl_fork();
						if(!$pid_new) {
							Logger::clog("Forked new GearmanWorker child", 1);
							createChild(array($this->childs[$pid]), 0);
						}
						$this->addPid($pid_new);
						unset($this->childs[$pid]);
						$this->childs[$pid_new] = $child;
						$this->deadChilds = array();
					}
				}
				$this->last_alive_check = time();
			}
			
			Logger::logrotate();
			sleep($this->interval);
		}
	}
	
	/**
	 * Function: doTask
	 *
	 * Führt jeden Übergebenen Task von $this->listener() aus und prüft ob der API Key richtig ist.
	 * Anschließend wird versucht die Übergebene Aufgabe auszuführen
	 *
	 * @param  string $task Der Task, welcher aus der CMD-Datei des phpNSTA ausgelesen wurde
	 * @return void
	 * @since 1.0
	 * @version 1.4
	 */
	function doTask($task){
		$TaskAsArray = explode(';', $task);
		if(!isset($TaskAsArray[1]) || !isset($TaskAsArray[2]) || $TaskAsArray[2] != Config::val('API', 'api_key')){
			Logger::log('ERROR: Invalide API command or wrong API key!', 0);
			return false;
		}
		switch($TaskAsArray[1]){
			case 'exit':
				pcntl_signal(SIGCHLD, SIG_IGN);
				if(Config::val('CPU', 'multi_kill')){
					foreach($this->known_pids as $pid){
						Logger::log('Killing my Child ['.$pid.']', 2);
						posix_kill($pid, SIGTERM);
					}
					foreach($this->known_pids as $pid){
						pcntl_waitpid($pid, $status);
						Logger::log('Child killed successfully ['.$pid.']', 1);
						$this->removePid($pid);
					}
					
				}else{
					foreach($this->known_pids as $pid){
						Logger::log('Killing my Child ['.$pid.']', 2);
						posix_kill($pid, SIGTERM);
						pcntl_waitpid($pid, $status);
						Logger::log('Child killed successfully ['.$pid.']', 1);
						$this->removePid($pid);
					}
				}
				
				if(Config::val('SSH', 'use_ssh_tunnel') === true){
					Logger::log("Terminating all Supervisor SSH connections", 0);
					$Supervisor = new Supervisor();
					$Supervisor->stop();
					$Supervisor->deleteConfiguration();
				}
				
				Logger::log("Harakiri! Astalavista baby...\n\n", 0);
				Boot::deletePID();
				Logger::close();
				exit(0);
				break;
				
			case 'export_started':
				Logger::log("openITCOCKPIT Export was started!", 0);
				foreach($this->known_pids as $pid){
					posix_kill($pid, EXPORT_STARTED);
				}
				break;
				
			case 'export_finished':
				Logger::log("openITCOCKPIT Export finished!", 0);
				foreach($this->known_pids as $pid){
					Logger::log("sending posix command", 0);
					posix_kill($pid, EXPORT_FINISHED);
				}
				break;
			
			default:
				Logger::log('ERROR: Unknown API command!', 0);
				break;
		}
	}
	
	/**
	 * Function: addPid
	 *
	 * Schreibt die übergebene PID (Integer oder Array mit mehreren PIDs) in $this->known_pids
	 * Alle bekannten PIDs kann der Parent-Prozess später überwachen und beenden
	 *
	 * @param int|array $pid  Der Dateipart zur Datei, für die die Rechte gesetzt werden sollen
	 * @return Boolean
	 * @since 1.0
	 *
	 */
	function addPid($pid=array()){
		if(is_array($pid) && !empty($pid)){
			foreach($pid as $p){
				$this->known_pids[] = $p;
			}
			return true;
		}
		
		if(!is_array($pid) && is_numeric($pid)){
			$this->known_pids[] = $pid;
			return true;
		}
		
		return false;
	}
	
	/**
	 * Function: removePid
	 *
	 * Löscht eine PID aus $this->known_pids
	 *
	 * @param int $pid2del Die zu löschende PID
	 * @return void
	 * @since 1.0
	 * @version 1.4
	 *
	 */
	function removePid($pid2del){
		if(in_array($pid2del, $this->known_pids)){
			$tmp_pids = $this->known_pids;
			$this->known_pids = array();
			foreach($tmp_pids as $pid){
				if($pid === $pid2del){
					continue;
				}
				$this->known_pids[] = $pid;
				unset($tmp_pids);
			}
		}
	}
	
	/**
	 * Function: addChilds
	 *
	 * Fügt alle Daten der aktuell laufenden Childs zu $this->childs hinzu
	 *
	 * @param array $ChildsAsArray  Daten die benötigten werden um ein Child zu erstellen
	 * @return void
	 * @since 1.4
	 *
	 */
	function addChilds($ChildsAsArray){
		$this->childs = $ChildsAsArray;
	}
	
	/**
	 * Function: sigchld_handler
	 *
	 * Überwacht ob ein Child Prozess beendet wird. Wenn ja wird dieser dann von $this->listener() neugestartet
	 * NUR WENN SSH UND SUPERVISOR GENUTZT WIRD!!!
	 *
	 * @param  int $signal Das Signal, welches der Linux Kernel an uns weiterleitet
	 * @return void
	 * @since 1.5.3
	 */
	function sigchld_handler($signal){
		//Get the dead child pid and clean up the zombie process
		$dead_child_pid = pcntl_wait($status, WNOHANG);
		// Dieser Child muss neu erstellt werden!
		if($dead_child_pid > 0){
			$this->removePid($dead_child_pid);
			
			//Prüft ob das Kind gestorben ist, oder planmäßig beendet wurde
			if(isset($this->plannedKill[$dead_child_pid])){
				unset($this->plannedKill[$dead_child_pid]);
			}
			Logger::log('Child died! ['.$dead_child_pid.'] detected by API::sigchld_handler()', 1);
			Logger::log('Giving the linux kernel a chance to cleanup bind sockets (waiting 1 secs)', 4);
			sleep(1);
			// Dem Kernel etwas Zeit geben und den Socket zu berreiningen, bevor ein neuer Worker erstellt wird
			foreach($this->childs as $pid=>$child){
				if($pid == $dead_child_pid){
					$this->deadChilds[$dead_child_pid] = $child;
				}
			}
		}
		pcntl_signal(SIGCHLD, array($this, 'sigchld_handler'));
	}
}
