<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Statische Klasse zum daemon handline
 *
 * Enthält alle wichtigen funktionen, die benötigt werden um den phpNSTA sauber zu starten
 * Zusätzlich ist diese Klasse für das Handling der PID des Hauptprozesses verantwortlich
 *
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @uses XMLRPCProxy, Klasse welche die Kommunikation mit Supervisor übernimmt
 * @usedby API
 * @usedby Boot
 * @usedby main.php
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class Supervisor{
	
	/**
	 * Function: __construct
	 *
	 * Erstellt das Object Supervisor
	 *
	 * @return object  Das Supervisor Object
	 * @since 1.4
	 *
	 */
	public function __construct(){
		$this->XMLRPC = new XMLRPCProxy("http://".Config::val('SUPERVISOR', 'username').":".Config::val('SUPERVISOR', 'password')."@".Config::val('SUPERVISOR', 'host').":".Config::val('SUPERVISOR', 'port')."/RPC2"); 
	}
	
	/**
	 * Function: writeConfiguration
	 *
	 * Erstellt die Supervisor Konfigurationsdatei für jedes SAT-System
	 *
	 * @param array $SattelitesAsArray  Ein Array mit der Konfiguration aller SAT-Systemen (SSH::$sat_systems)
	 * @return void
	 * @since 1.4
	 * @version 1.7.3
	 *
	 */
	public function writeConfiguration($SattelitesAsArray){
		$prefix = Config::val('SUPERVISOR', 'prefix');
		$suffix = Config::val('SUPERVISOR', 'suffix');
		foreach($SattelitesAsArray as $SATAsArray){
			Logger::log("Creating Supervisor configuration for: ".$SATAsArray['name']);
			$CleanName = $this->clearName($SATAsArray['name']);
			$file = fopen(Config::val('SUPERVISOR', 'config_path').$prefix.$CleanName.$suffix, 'w+');
			fwrite($file, $this->retrunHeader());
			fwrite($file, "[program:".$CleanName."]\n");
			fwrite($file, "command = ssh -NnC -L 127.0.0.1:".$SATAsArray['local_port'].":".Config::val('SSH', 'local_bind_address').":".Config::val('SSH', 'remote_port')." -p ".Config::val('SSH', 'port')." -i ".Config::val('SSH', 'private_path')." -o StrictHostKeyChecking=no -o ExitOnForwardFailure=yes -o ConnectTimeout=10 -o ServerAliveInterval=15 ".Config::val('SSH', 'username')."@".$SATAsArray['ip']."\n");
			fwrite($file, "autostart = true\n");
			fwrite($file, "startsecs = 11\n");
			fwrite($file, "numprocs = 1\n");
			fwrite($file, "startretries = 800000000\n");
			fwrite($file, "redirect_stderr = true\n");
			fwrite($file, "stdout_logfile = /tmp/supervisor_".$CleanName.".log\n");
			fwrite($file, "stdout_logfile_maxbytes = 50MB\n");
			fwrite($file, "stdout_logfile_backups = 2\n");
			fclose($file);
			$this->XMLRPC->supervisor->addProcessGroup($SATAsArray['name']);
		}
	}
	
	/**
	 * Function: deleteConfiguration
	 *
	 * Löscht vom phpNSTA erstellten Supervisor-Konfigurationsdateien
	 *
	 * @return void
	 * @since 1.4
	 * @version 1.7.3
	 *
	 */
	public function deleteConfiguration(){
		Logger::log("Deleting Supervisor configuration");
		$prefix = Config::val('SUPERVISOR', 'prefix');
		foreach(glob(Config::val('SUPERVISOR', 'config_path').$prefix.'*.conf') as $SupervisorConfigFile){
			unlink($SupervisorConfigFile);
		}
	}
	
	/**
	 * Function: reload
	 *
	 * Führt einen reload der Konfigurationsdateien von Supervisor aus
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public function reload(){
		$this->XMLRPC->supervisor->reloadConfig();
	}
	
	/**
	 * Function: restart
	 *
	 * Führt einen Neustart von Supervisor über die XML RPC API aus
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public function restart(){
		$this->XMLRPC->supervisor->restart();
	}
	
	/**
	 * Function: start
	 *
	 * Startet alle Prozesse, welche von Supervisor kontrolliert werden
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public function start(){
		$this->XMLRPC->supervisor->startAllProcesses(true);
	}
	
	/**
	 * Function: stop
	 *
	 * Beendet alle laufenden Prozesse, welche von Supervisor kontroliert werden
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public function stop(){
		$this->XMLRPC->supervisor->stopAllProcesses();
	}
	
	/**
	 * Function: start_process
	 *
	 * Startet einen Prozesse, welche von Supervisor kontrolliert werden
	 *
	 * @param string $proccess_name  Name des Prozesses, welcher gestartet werden soll
	 * @return void
	 * @since 1.4
	 *
	 */
	public function start_process($proccess_name){
		$this->XMLRPC->supervisor->startProcess($proccess_name);
	}
	
	/**
	 * Function: stop_process
	 *
	 * Beendet einen laufenden Prozesse, welche von Supervisor kontroliert werden
	 *
	 * @param string $proccess_name  Name des Prozesses, welcher beendet werden soll
	 * @return void
	 * @since 1.4
	 * @todo Not supported by Supervisor yet! 
	 *
	 */
	public function stop_process($proccess_name){
		// Not implementet yet!
	}
	
	/**
	 * Function: listMethods
	 *
	 * Listet alle Funktionen der XML-RPC API von Supervisor auf
	 *
	 * @param bool $return   Entscheidet ob der Rückgabewert zurück gegeben werden soll, oder in das Logfile geschrieben wird
	 * @return mixed         Gibt den Rückgabewert der XML-RPC API zurück
	 * @since 1.4
	 *
	 */
	public function listMethods($return = false){
		if($return){
			return $this->XMLRPC->system->listMethods();
		}else{
			Logger::dump($this->XMLRPC->system->listMethods());
		}
	}
	
	/**
	 * Function: test
	 *
	 * Prüft ob auf die XML-RPC API von Supervisor zugegriffen werden kann
	 *
	 * @return Boolean
	 * @since 1.4
	 *
	 */
	public function test(){
		$return = $this->listMethods(true);
		$this->valReturn($return);
		if(is_array($return) && !empty($return)){
			return true;
		}
		return false;
	}
	
	/**
	 * Function: valReturn
	 *
	 * Von der Supervisor XMLRPC API muss ein array zurück kommen!
	 * Ist dies nicht der Fall, hat mit der Verbindung etwas nicht geklappt
	 *
	 * @param array $return  Ergebnis der Supervisor XMLRCP API Abfrage
	 * @return Boolean
	 * @since 1.4
	 *
	 */
	public function valReturn($return){
		if(is_array($return)){
			return true;
		}
		Logger::log("Error while receiving data from Supervisors XML RPC API" ,0);
		Logger::dump($return);
	}
	
	/**
	 * Function: sleep
	 *
	 * Gibt dem Supervisor etwas Zeit, bis die Änderungen wirksam wurden
	 * Oder um die einzelnen Prozesse zu starten (SSH-Verbindungen aufbauen)
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public function sleep(){
		Logger::log("Waiting for Supervisor (".Config::val('SUPERVISOR', 'sleep')."secs)", 5);
		sleep(Config::val('SUPERVISOR', 'sleep'));
	}
	
	
	/**
	 * Function: clearName
	 *
	 * ERsetzt alle Sonderzeichen und einen Unterstrich (_)
	 *
	 * @param  string $name   Der String indem alle Sonderzeichen entfernt werden sollen
	 * @return string         Der Name des SAT-Systems ohne Sonderzeichen
	 * @since 1.4
	 *
	 */
	public function clearName($name){
		return preg_replace('/[^-a-zA-Z0-9_\.]/','_',$name);
	}
	
	/**
	 * Function: retrunHeader
	 *
	 * Gibt den Header zurück, welcher in jede Supervisor Konfigurationsdatei geschreiben wird
	 * die durch den phpNSTA erstellt wird.
	 *
	 * @return string  Der Header für die Spervisor Konfiguration
	 * @since 1.4
	 *
	 */
	public function retrunHeader(){
		return 
			"; Dynamic supervisor configuration file
; Created ".date(Config::val('LOG', 'date_format'))." by phpNSTA
;     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN\n";
	}
	
	/**
	 * Function: checkConnection
	 *
	 * Prüft für einen Angegebenen Prozessnamen, ob der SSH-Tunnel laut Supervisor aufgebaut ist, oder nicht
	 *
	 * @param string $process_name  Der Name des Prozess, welcher überprüft werden soll
	 * @return bool True = Verbindung Ok, False Verbindung nicht ok
	 * @since 1.4
	 *
	 */
	public function checkConnection($process_name){
		$result = $this->XMLRPC->supervisor->getProcessInfo($this->clearName($process_name));
		if($result['statename'] != 'RUNNING'){
			return false;
		}
		return true;
	}
}
