<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */


require_once "phpVersion.php";
require_once "class.cli.php";
require_once "static.class.config.php";
require_once "static.class.nagios.php";
require_once "static.class.sat.php";
require_once "static.class.log.php";
require_once "class.api.php";
require_once "static.class.ssh.php";
require_once "class.xmlrpc.php";
require_once "class.supervisor.php";
require_once "static.class.tsync.php";

//Loading openITCOCKPIT Core-Libs
if(Config::val('MAIN', 'oitc_version') == 2){
	require_once(Config::val('MAIN', 'oitc_core'));
}else{
	//Loading CakePHPs database configuration
	define('ENVIRONMENT', 'production');
	require_once(Config::val('MAIN', 'oitc_database'));
	//openITCOCKPIT V3 /etc/openitcockpit/app/Config/database.php
	$DBConfig = new DATABASE_CONFIG();
	//Loading V2 MySQL Class
	require_once "inc.class.mysql.php";
	
	/** @name object $DB  Globale Variable mit dem Datenbank Object für SELECT Abfragen */
	$DB = new MySQLHandler($DBConfig->default['host'], 3306, $DBConfig->default['database'], $DBConfig->default['login'], $DBConfig->default['password']);
	
}



/**
 * Statische Klasse zum daemon handline
 *
 * Enthält alle wichtigen funktionen, die benötigt werden um den phpNSTA sauber zu starten
 * Zusätzlich ist diese Klasse für das Handling der PID des Hauptprozesses verantwortlich
 *
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @uses SSH - Statische Klasse hält ein array mit allen SAT-Systemen
 * @uses Supervisor, Klasse welche die Kommunikation mit Supervisor übernimmt
 * @uses CLI, Klasse zur Ausgabe farbiger Texte auf der Konsole
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class Boot{

	/**
	 * Function: bootup
	 *
	 * Führt alle, für den Start des phpNSTA wichtigen Routinen aus.
	 * Gibt des weiteren nützliche Informationen dem Benutzer aus.
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function bootup(){
		Logger::welcome();
		$CLI = new CLI();
		$CLI->echoWelcome();
		Logger::log('preparing to launch phpNSTA' ,0);
		$CLI->_echo('preparing to launch phpNSTA', 'light_blue');
		SAT::fetch();
		Logger::log('Wiping API command file', 0);
		$API = new API();
		$API->wipe();
		unset($API);
		define('EXPORT_STARTED', SIGHUP);
		define('EXPORT_FINISHED', SIGINT);
		Logger::log('I found '.SAT::$count.' satellite systems', 0);
		$CLI->_echo('I found '.SAT::$count.' satellite systems', 'light_blue');
		SAT::createCMDFile();
		if(Config::val('CPU', 'multithreading')){
			Logger::log('Multithreading is on', 0);
			Logger::log('Max CPU-Threads: '.Config::val('CPU', 'max_threads'), 0);
			$CLI->_echo('Multithreading is on', 'green');
			$CLI->_echo('Max CPU-Threads: '.Config::val('CPU', 'max_threads'), 'light_blue');
		}else{
			Logger::log('Multithreading is off', 0);
			$CLI->_echo('Multithreading is off', 'red');
		}
		$CLI->_echo('My Logfile is: '.Config::val('LOG', 'logfile'), 'light_blue');
		ini_set('error_log', Config::val('LOG', 'logfile'));

		if(!self::checkPID()){
			$CLI->_echo('Finished daemonizing... [My PID = '.getmypid().']', 'green');
			Logger::log('Finished daemonizing... [My PID = '.getmypid().']', 0);
			self::createPID();
		}else{
			$CLI->_echo('ERROR! PID FILE '.Config::val('MAIN', 'pid_file').' already exists! I will die now!', 'red');
			Logger::log('ERROR! PID FILE '.Config::val('MAIN', 'pid_file').' already exists! I will die now!', 0);
			exit(3);
		}
		
	}
	
	/**
	 * Function: checkPID
	 *
	 * Prüft ob das PID-File des phpNSTA bereits existiert
	 *
	 * @return Boolean
	 * @since 1.0
	 *
	 */
	public static function checkPID(){
		if(file_exists(Config::val('MAIN', 'pid_file'))){
			return true;
		}
		
		return false;
	}
	
	/**
	 * Function: createPID
	 *
	 * Erstellt das PID-File des phpNSTA und füllt es mit der aktuellen PID des Parent-Prozesses
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function createPID(){
		$file = fopen(Config::val('MAIN', 'pid_file'), 'w+');
		fwrite($file, getmypid());
		fclose($file);
	}
	
	/**
	 * Function: deletePID
	 *
	 * Prüft ob das PID-File existiert und löscht es anschließend
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function deletePID(){
		if(file_exists(Config::val('MAIN', 'pid_file'))){
			unlink(Config::val('MAIN', 'pid_file'));
		}
	}
	
	/**
	 * Function: selfTest
	 *
	 * Führt einen Selbstest aus, und prüft ob wichtige Dateien existieren und beschreibbar sind.
	 *
	 * @return Boolean
	 * @since 1.0
	 * @version 1.4.4
	 *
	 */
	public static function selfTest(){
		$error = false;
		$CLI = new CLI();
		
		//Checking for path to nagios.cmd on the master system
		if(!is_dir(Config::val('SAT', 'master_cmd_path')) || !is_writable(Config::val('SAT', 'master_cmd_path'))){
			$error = true;
			$CLI->_echo("ERROR: ".Config::val('SAT', 'master_cmd_path')." does not exist or is not writable!", 'red');
		}
		
		if(strlen(Config::val('NAGIOS', 'grep_for_nagios')) > 0){
			exec(Config::val('NAGIOS', 'grep_for_nagios'), $output);
			if(empty($output) || !is_array($output)){
				$error = true;
				$CLI->_echo("ERROR: Nagios is not running!!!", 'red');
			}
		}
		//Checking the nagios.cmd on the master system
		if(!is_writable(Config::val('NAGIOS', 'cmd_file'))){
			//Auskommentiert, um Nagios bei grossen Systemen bzw laengerer Ladezeit starten zu koennen			
			//$error = true;
			$CLI->_echo("ERROR: ".Config::val('NAGIOS', 'cmd_file')." does not exist or is not writable!", 'red');
		}
		
		//Checking nagios spool dir
		if(!is_dir(Config::val('NAGIOS', 'spool_dir')) || !is_writable(Config::val('NAGIOS', 'spool_dir'))){
			$error = true;
			$CLI->_echo("ERROR: ".Config::val('NAGIOS', 'spool_dir')." does not exist or is not writable!", 'red');
		}
		
		//Checking my logrotate path
		if(!is_dir(Config::val('LOG', 'logrotate_path'))){
			$error = true;
			$CLI->_echo("ERROR: ".Config::val('LOG', 'logrotate_path')." does not exist", 'red');
		}
		
		//Checking the SSH Settings
		if(Config::val('SSH', 'use_ssh_tunnel') === true){
			$PortRangeToCheckAsArray = SSH::getPortrange();
			if($PortRangeToCheckAsArray['start'] <= 0 || $PortRangeToCheckAsArray['end'] <= 0 || $PortRangeToCheckAsArray['end'] <= $PortRangeToCheckAsArray['start']){
				$error = true;
				$CLI->_echo("ERROR: Your configured port range is not a valid value: ".Config::val('SSH', 'port_range'), 'red');
			}
		}
		
		//Checking SSH port range
		if(Config::val('SSH', 'use_ssh_tunnel') === true){
			
			if(!function_exists("curl_init")){
				$error = true;
				$CLI->_echo("ERROR:  Call to undefined function curl_init - try apt-get install php5-curl", 'red');
			}
			
			if(!function_exists("ssh2_connect")){
				$error = true;
				$CLI->_echo("ERROR:  Call to undefined function ssh2_connect - try apt-get install libssh2-php", 'red');
			}
			
			if(!file_exists(Config::val('SSH', 'private_path'))){
				$error = true;
				$CLI->_echo("ERROR: Private key file for SSH connection does not exists", 'red');
			}
			
			if(!file_exists(Config::val('SSH', 'public_path'))){
				$error = true;
				$CLI->_echo("ERROR: Public key file for SSH connection does not exists", 'red');
			}
			
			if(!is_dir(Config::val('SUPERVISOR', 'config_path'))){
				$error = true;
				$CLI->_echo("ERROR: Path for Supervisors configuration files does not exists", 'red');
			}
			
			$Supervisor = new Supervisor();
			if(!$Supervisor->test()){
				$error = true;
				$CLI->_echo("ERROR: Can't connect to Supervisors XML RPC API!", 'red');
				$CLI->_echo('Notice: If you use a proxy set no_proxy="127.0.0.1" in /etc/environment and restart your SSH session', 'light_blue');
			}
			
			if($PortRangeToCheckAsArray['available_ports'] <= 0){
				$error = true;
				$CLI->_echo("ERROR: Your configured port range is not a valid value: ".Config::val('SSH', 'port_range'), 'red');
			}
			
			if(Config::val('CPU', 'multithreading') === true){
				if($PortRangeToCheckAsArray['available_ports'] < Config::val('CPU', 'max_threads')){
					$error = true;
					$CLI->_echo("ERROR: Your configured more CPU threads(".Config::val('CPU', 'max_threads').") than ports(".$PortRangeToCheckAsArray['available_ports'].") for the SSH connection. Please increase your port range in config.php", 'red');
				}
			}
		
			if(Config::val('SSH', 'check_connection') === true){
				//Run connection check for each satellite system
				SSH::checkAllConnections();
			}
		
		}
		
		//Checking TSYNC settings
		if(Config::val('TSYNC', 'synchronize_time') === true){
			if(!in_array(Config::val('TSYNC', 'every'), array('hour', 'minute', 'day'))){
				$error = true;
				$CLI->_echo("ERROR: Your configured value(".Config::val('TSYNC', 'every').") for TSYNC Daemon is not supported", 'red');
			}
		}
		
		//Checking Nagios-4x
		if(Config::val('NAGIOS', 'nagios4x_compatibility') === true){
			if(!file_exists(Config::val('NAGIOS', 'nagiostats'))){
				$error = true;
				$CLI->_echo("ERROR: Could not found ".Config::val('NAGIOS', 'nagiostats'), 'red');
			}
			
			exec(Config::val('NAGIOS', 'nagiostats').' -m -d NAGIOSVERSION', $nagios_version);
			if(isset($nagios_version[0])){
				Logger::log("Found Nagios version: ".$nagios_version[0], 0);
				$CLI->_echo("Found Nagios version: ".$nagios_version[0], 'light_blue');
			}else{
				Logger::log("Unable to detect Nagios version", 0);
				$CLI->_echo("ERROR: Unable to detect Nagios version", 'red');
			}
			unset($nagios_version);
			
			exec(Config::val('NAGIOS', 'nagiostats').' -m -d PROGRUNTIME', $nagios_runtime_human);
			if(isset($nagios_runtime_human[0])){
				Logger::log("Nagios runtime: ".$nagios_runtime_human[0], 0);
				$CLI->_echo("Nagios runtime: ".$nagios_runtime_human[0], 'light_blue');
			}else{
				Logger::log("Unable to detect Nagios runtime, your phpNSTA may be never start with his work!", 0);
				$CLI->_echo("ERROR: Unable to detect Nagios runtime, your phpNSTA may be never start with its work!", 'red');
			}
			unset($nagios_runtime_human);
			
		}
		
		if($error === true){
			$CLI->_echo("ERROR: Pre-flight check not passed. Check your configuration and file permissions", 'red');
		}else{
			$CLI->_echo("Pre-flight check passed\t\t\t\tok", 'green');
		}
		
		return $error;
	}
}
