<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Statische Klasse für die Kommunikation mit Nagios/Naemon
 *
 * Diese Klasse bietet jedem SAT-System die Möglichkeit, einen Cache mit Ergebnissen zu erstellen.
 * Wenn das maximale Limit des Caches erreicht ist, oder das Verfallsdatum wurde überschritten,
 * wird dieser an Nagios oder Naemon übergeben.
 *
 * Für die Übergabe stehen 3 Wege zur Verfügung:
 *   - checkresults -> Ist ein Verzeichniss von Nagios in dem Host und Services Ergebnisse von Nagios
 *                      gespeichert werden. (Nicht mit nagios4 oder naemon nutzen!)
 *   - nagios.cmd   -> Über das Nagios External Command file, können beliebige Ergebnisse an Nagios übergeben werden
 *                     hier exstiert allerdings keine Flusskontrolle und es kann passieren, das der phpNSTA die nagios.cmd
 *                     als eine Textdatei erstellt, was zu Problemen beim Neustart von Nagios führt!
 *   - nagios.qh    -> Hier wird der Query Handler von Nagios verwendet. Dies ist die beste Möglichkeit und funktioniert
 *                     im Moment nur mit Naemon. Der Query Handler bietet eine Flusskontrolle und kann Erfolgs- und
 *                     Fehlermeldungen ausgeben.
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @usedby main.php
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class Nagios{
	
	/** @var array $cache Array mit dem Cache der Ergebnisse jedes SAT-Systems */
	public static $cache = array();
	
	/** @var int $last_change Enthält den Timestamp, wann der cache das letzte mal an Nagios übergeben wurde */
	public static $last_change = 0;
	
	/**
	 * @var bool $export_running Ob gerade ein Export in openITCOCKPIT V2 ausgeführt wird oder nicht
	 * @deprecated Sollte nicht mehr verwendet werden, da es nicht mehr benötigt wird!
	 */
	public static $export_running = false;
	
	/**
	 * @var bool $nagios_runtime_ok Ob nagios schon länger als X Sekunden läuft
	 * @deprecated Nagios4 workaround, sollte nicht mehr verwendet werden!!!
	 */
	public static $nagios_runtime_ok = true;
	
	public static $MOD_GEARMAN_ENCRYPTION = true;
	
	public static $MOD_GEARMAN_KEY = '';
	
	
	
	/**
	 * Function: Cache
	 *
	 * Erstellt ein Array als Cache, damit mehrere Ergebnisse auf einmal an Nagios übergeben werden, und
	 * nicht jedes Ergebnis einzeln
	 * Hintergrund: Größere Dateien (wichtig bei checkresults dir) können schneller verarbeitet werden
	 *              Die maximale Größe des Cache sollte 200 nicht überschreiten. Bei einem größeren oder kleineren Cache
	 *              kann es passieren, das der phpNSTA/Nagios/Naemon langsamer arbeitet.
	 *              200 hat sich als guter Wert erwiesen.
	 *
	 * Wenn der Cache voll ist (Config::val('NAGIOS', 'max_checks') erreicht hat), dann werden die Ergebnisse von dieser
	 * Funktion direkt an Nagios übergeben!
	 * 
	 * @param array $checkresults  Ein Array mit den Ergebnissesn der SAT-Systeme
	 * @return void
	 * @todo Für die Methoden Cache() und processedCache() sollte eine eigene Funktion erstellt werden, welche die Übergabe an Nagios übernimmt
	 * @since 1.0
	 * @version 1.7.6
	 *
	 */
	public static function Cache($checkresults = array()){
		if(self::$last_change == 0){
			self::$last_change = time();
		}
		self::$cache[] = $checkresults[0];
		unset($checkresults[0]);
		if((sizeof(self::$cache) >= Config::val('NAGIOS', 'max_checks') || self::$last_change < (time() - Config::val('NAGIOS', 'max_wait'))) && self::$export_running == false && self::$nagios_runtime_ok === true){
			Logger::clog('Hand over my cache to Nagios [Cachesize: '.sizeof(self::$cache).' / '.Config::val('NAGIOS', 'max_checks').']', 3);
			if(Config::val('NAGIOS', 'use_spooldir') === 1){
				//use nagios spool dir
				self::buildFile(self::$cache);
			}elseif(Config::val('NAGIOS', 'use_spooldir') === 2){
				//use nagios.cmd
				foreach(self::$cache as $ExternalCommandAsArray){
					$NagiosCMD = self::openCMD();
					self::toNagiosCMD($ExternalCommandAsArray, $NagiosCMD);
					fclose($NagiosCMD);
				}
			}elseif(Config::val('NAGIOS', 'use_spooldir') === 3){
				foreach(self::$cache as $ExternalCommandAsArray){
					$NagiosQH = self::openQH();
					self::toNagiosQh($ExternalCommandAsArray, $NagiosQH);
					socket_close($NagiosQH);
				}
			}elseif(Config::val('NAGIOS', 'use_spooldir') === 4){
				$Mod_Gearman_GearmanClient = self::getModGearmanGearmanClient();
				foreach(self::$cache as $ExternalCommandAsArray){
					self::toModGearman($ExternalCommandAsArray, $Mod_Gearman_GearmanClient);
				}
				unset($Mod_Gearman_GearmanClient);
			}
			self::$cache = array();
			self::$last_change = time();
			return;
		}
	}
	
	/**
	 * Function: processedCache
	 *
	 * Ist eine !force! Methode um die Verarbeitung des Caches zu erzwingen.
	 * Wird zum Beispiel aufgerufen, wenn der phpNSTA beendet wird
	 *
	 * Wie self::Cache() werden die Ergebnisse aus self::$cache direkt an Nagios/Naemon übergeben
	 *
	 * @param  bool $force  Soll der phpNSTA warten, bis die Datei von Nagios verarbeitet wurde. (Einstellbar in der config.php)! Dieser Parameter ist nur für den Stop des phpNSTA! (Default: false)
	 * @param  bool $max_wait_timeout  Handelt es sich um einen max_wait Timeout aus der config.php? (Default: false)
	 * @return void
	 * @todo Für die Methoden Cache() und processedCache() sollte eine eigene Funktion erstellt werden, welche die Übergabe an Nagios übernimmt
	 * @since 1.0
	 * @version 1.7.6
	 *
	 */
	public static function processedCache($force = false, $max_wait_timeout = false){
		if(!empty(self::$cache)){
			if($max_wait_timeout === false){
				Logger::log('Force processing of cached data because of SIGTERM',1);
			}
			if(Config::val('NAGIOS', 'use_spooldir') === 1){
				//use nagios spool dir 
				self::buildFile(self::$cache, $force);
			}elseif(Config::val('NAGIOS', 'use_spooldir') === 2){
				//use nagios.cmd
				foreach(self::$cache as $ExternalCommandAsArray){
					$NagiosCMD = self::openCMD();
					self::toNagiosCMD($ExternalCommandAsArray, $NagiosCMD);
					fclose($NagiosCMD);
				}
			}elseif(Config::val('NAGIOS', 'use_spooldir') === 3){
				foreach(self::$cache as $ExternalCommandAsArray){
					$NagiosQH = self::openQH();
					self::toNagiosQh($ExternalCommandAsArray, $NagiosQH);
					socket_close($NagiosQH);
				}
			}elseif(Config::val('NAGIOS', 'use_spooldir') === 4){
				$Mod_Gearman_GearmanClient = self::getModGearmanGearmanClient();
				foreach(self::$cache as $ExternalCommandAsArray){
					self::toModGearman($ExternalCommandAsArray, $Mod_Gearman_GearmanClient);
				}
				unset($Mod_Gearman_GearmanClient);
			}
			self::$cache = array();
			self::$last_change = time();
		}
	}
	
	
	/**
	 * Function: toNagiosCMD
	 *
	 * Übergibt die gesammelten Daten an Nagios und nutzt dazu die nagios.cmd anstelle des spool_dir
	 *
	 * @param  array $ExternalCommandAsArray  Das external Command für Nagios als Array
	 * @param  resource $NagiosCMD Die nagios.cmd als PHP-Resource (fopen)
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function toNagiosCMD($ExternalCommandAsArray = array(), $NagiosCMD){
		if(is_array($ExternalCommandAsArray) && !empty($ExternalCommandAsArray) && is_resource($NagiosCMD)){
			switch($ExternalCommandAsArray['TYPE']){
				case 'SERVICE':
					fwrite($NagiosCMD, '['.$ExternalCommandAsArray['TIMET'].'] PROCESS_SERVICE_CHECK_RESULT;'.$ExternalCommandAsArray['HOSTNAME'].';'.$ExternalCommandAsArray['SERVICEDESC'].';'.$ExternalCommandAsArray['SERVICESTATEID'].';'.$ExternalCommandAsArray['SERVICEOUTPUT'].' | '.$ExternalCommandAsArray['SERVICEPERFDATA']."\n");
					break;
				
				case 'HOST':
					fwrite($NagiosCMD, '['.$ExternalCommandAsArray['TIMET'].'] PROCESS_HOST_CHECK_RESULT;'.$ExternalCommandAsArray['HOSTNAME'].';'.$ExternalCommandAsArray['HOSTSTATEID'].';'.$ExternalCommandAsArray['HOSTOUTPUT']."\n");
					break;
			}
		}
	}
	
	/**
	 * Function: openCMD
	 *
	 * Öffnet die nagios.cmd wenn diese existiert
	 *
	 * @return  resource  Die nagios.cmd als PHP-Resource
	 * @since 1.5.4
	 *
	 */
	public static function openCMD(){
		$NagiosCMD = false;
		while(true){
			clearstatcache();
			if(file_exists(Config::val('NAGIOS', 'cmd_file'))){
				$NagiosCMD = fopen(Config::val('NAGIOS', 'cmd_file'), 'a+');
				stream_set_timeout($NagiosCMD, 1);
				stream_set_blocking($NagiosCMD, 0);
				return $NagiosCMD;
			}else{
				Logger::clog('Can not open cmd file: '.Config::val('NAGIOS', 'cmd_file'), 3);
			}
			pcntl_signal_dispatch();
			sleep(5);
		}
	}
	
	/**
	 * Function: openQH
	 *
	 * Öffnet den nagios query handler
	 *
	 * @return  resource  Die nagios.qh als PHP-Resource
	 * @since 1.5.4
	 *
	 */
	public static function openQH(){
		$NagiosQH = false;
		while(true){
			//$NagiosQH = fsockopen('unix://'.Config::val('NAGIOS', 'qh_file'), NULL, $errno, $errstr, 5);
			//$NagiosQH = stream_socket_client('unix://'.Config::val('NAGIOS', 'qh_file'), $errno, $errstr, 5);

			$NagiosQH = socket_create(AF_UNIX, SOCK_STREAM, 0);
			$connectionResult = socket_connect($NagiosQH, Config::val('NAGIOS', 'qh_file'));
			
			//socket_set_nonblock($NagiosQH);
			
			
			if($connectionResult === false){
				$errno = socket_last_error($NagiosQH);
				$errstr = socket_strerror($errno);			

				Logger::clog('Can not open query handler: '.Config::val('NAGIOS', 'qh_file'), 3);
				if($errno){
					Logger::clog('[Error: '.$errno.'] '.$errstr, 3);
				}
				pcntl_signal_dispatch();
				sleep(5);
			}else{
				//Socket erfolgreich geöffnet
				//socket_connect($NagiosQH, Config::val('NAGIOS', 'qh_file'));
				socket_set_nonblock($NagiosQH);
				return $NagiosQH;
			}
		}
	}
	
	/**
	 * Function: toNagiosQh
	 *
	 * Übergibt die gesammelten Daten den query handler von nagios4/naemon
	 *
	 * @param  array $ExternalCommandAsArray  Das external Command für Nagios als Array
	 * @param  resource $NagiosQH  Die nagios.qh als PHP-Resource
	 * @return bool       true wenn die verarbeitung geklappt hat, false wenn nicht
	 * @since 1.5.4
	 *
	 */
	public static function toNagiosQh($ExternalCommandAsArray = array(), $NagiosQH){
		if(is_array($ExternalCommandAsArray) && !empty($ExternalCommandAsArray) && is_resource($NagiosQH)){
			$query = "#command run ";
			switch($ExternalCommandAsArray['TYPE']){
				case 'SERVICE':
					$query .= '['.$ExternalCommandAsArray['TIMET'].'] PROCESS_SERVICE_CHECK_RESULT;'.$ExternalCommandAsArray['HOSTNAME'].';'.$ExternalCommandAsArray['SERVICEDESC'].';'.$ExternalCommandAsArray['SERVICESTATEID'].';'.$ExternalCommandAsArray['SERVICEOUTPUT'].' | '.$ExternalCommandAsArray['SERVICEPERFDATA'];
					break;
				
				case 'HOST':
					$query .= '['.$ExternalCommandAsArray['TIMET'].'] PROCESS_HOST_CHECK_RESULT;'.$ExternalCommandAsArray['HOSTNAME'].';'.$ExternalCommandAsArray['HOSTSTATEID'].';'.$ExternalCommandAsArray['HOSTOUTPUT'];
					break;
			}
			if(isset($ExternalCommandAsArray['LONGSERVICEOUTPUT']) && strlen($ExternalCommandAsArray['LONGSERVICEOUTPUT']) > 0){
				$query .= PHP_EOL.$ExternalCommandAsArray['LONGSERVICEOUTPUT'].PHP_EOL;
			}

			$query .= "\0";

			socket_write($NagiosQH, $query);

			/*for($written = 0; $written < strlen($query); $written += $len) {
				$len = fwrite($NagiosQH, substr($query, $written));
					if($len === false)
						Logger::clog('Can not write to nagios.qh!');
			}*/
$response = "";
			
			/*
		        while(($c = socket_read($NagiosQH,1))!==false) {
		                if ($response && ($c === "" || $c === "\0"))
		                        break;
		                $response .= $c;
		        }*/

			//$response = trim(stream_get_contents($NagiosQH));


			while(true){
				$read = array($NagiosQH);
				$write = NULL;
				$except = NULL;
				if(socket_select($read, $write, $except, 0) < 1){
					continue;
				}
				if(in_array($NagiosQH, $read)){
					break;
				}	
			}
			$response = trim(socket_read($NagiosQH, 1024, PHP_NORMAL_READ));
			
			//socket_recv($NagiosQH, $buf, 7, MSG_WAITALL);
			//$response = $buf;

			if($response != '200: OK'){
				Logger::clog('Data not processed successfully! Response: '.$response, 3);
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Function: buildFile
	 *
	 * Ist für die Übergabe der Daten an Nagios verantwortlich.
	 *
	 * @param array $results  Die Ergebnisse, welche in ein Checkresults-File geschrieben werden sollen
	 * @param bool $force     Wenn true, wird nicht gewartet, bis Nagios die Checkresult-Datei verarbeitet hat (Default: false)
	 * @return void
	 * @since 1.0
	 * @version 1.4.4
	 *
	 */
	public static function buildFile($results = array(array()), $force = false){
		if(is_array($results) && !empty($results)){
			$filename = Config::val('NAGIOS', 'spool_dir').self::generateFilename();
			$file = fopen($filename, 'w+');
			foreach($results as $params){
				switch($params['TYPE']){
					case 'SERVICE':
						fwrite($file, "### Passive Check Result File ###\n");
						fwrite($file, "file_time=".time()."\n\n");
						fwrite($file, "### phpNSTA-Injection ###\n");
						fwrite($file, "# Time: ".$params['TIMET']."\n");
						fwrite($file, "host_name=".$params['HOSTNAME']."\n");
						fwrite($file, "service_description=".$params['SERVICEDESC']."\n");
						fwrite($file, "check_type=1\n");
						fwrite($file, "early_timeout=0\n");
						fwrite($file, "exited_ok=1\n");
						fwrite($file, "start_time=".$params['TIMET'].".0\n");
						fwrite($file, "finish_time=".$params['TIMET'].".0\n");
						fwrite($file, "return_code=".$params['SERVICESTATEID']."\n");
						fwrite($file, "output=".$params['SERVICEOUTPUT']." | ".$params['SERVICEPERFDATA']."\n\n");
						break;
					
					case 'HOST':
						fwrite($file, "### Passive Check Result File ###\n");
						fwrite($file, "file_time=".time()."\n\n");
						fwrite($file, "### phpNSTA-Injection ###\n");
						fwrite($file, "# Time: ".$params['TIMET']."\n");
						fwrite($file, "host_name=".$params['HOSTNAME']."\n");
						fwrite($file, "check_type=1\n");
						fwrite($file, "early_timeout=0\n");
						fwrite($file, "exited_ok=1\n");
						fwrite($file, "start_time=".$params['TIMET'].".0\n");
						fwrite($file, "finish_time=".$params['TIMET'].".0\n");
						fwrite($file, "return_code=".$params['HOSTSTATEID']."\n");
						fwrite($file, "output=".$params['HOSTOUTPUT']."\n\n");
						break;
					
					default:
						Logger::clog('Unknown Data for Nagios:',3);
						Logger::dump($params);
						return;
				}
			}
			fclose($file);
			self::fixPermissions($filename);
			touch($filename.".ok");
			self::fixPermissions($filename.".ok");
			if(Config::val('NAGIOS', 'waid_for_nagios') === true && $force === false){
				while(true){
					if(!file_exists($filename)){
						break;
					}
					pcntl_signal_dispatch();
					sleep(1);
				}
			}
		}
	}
	
	/**
	 * Function: fixPermissions
	 *
	 * Setzt die benötigten Dateirechte, damit Nagios die Datei einlesen und verarbeiten kann
	 *
	 * @param  string $filename  Der Dateipart zur Datei, für die die Rechte gesetzt werden sollen
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function fixPermissions($filename){
		if(file_exists($filename)){
			chown($filename, Config::val('NAGIOS', 'nagios_user'));
			chgrp($filename, Config::val('NAGIOS', 'apache_group'));
			chmod($filename, Config::val('NAGIOS', 'chmod'));
		}
	}
	
	/**
	 * Function: filename
	 *
	 * Erzeigt einen eindeutigen Dateinamen
	 *
	 * @return string  Der erstellte Dateiname
	 * @since 1.0
	 *
	 */
	public static function filename(){
		$filename = generateFilename();
		while(in_array($filename, scandir(Config::val('NAGIOS', 'spool_dir')))){
			$filename = self::generateFilename();
		}
		
		return $filename;
		
	}
	
	/**
	 * Function: generateFilename
	 *
	 * Ist für die Erstellung des zufälligen Namens verantworlich
	 *
	 * @return string  Der erzeugte Dateiname
	 * @since 1.0
	 *
	 */
	public static function generateFilename(){
		$char = array(0, 1, 2, 3, 4, 5, 6, 7, 8 ,9, 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z');
		$size = (sizeof($char)-1);
		$filename = Config::val('NAGIOS', 'soolfile_prefix');
		for($i = 0; $i < Config::val('NAGIOS', 'spoolfile_random_length'); $i++){
			$filename.=$char[rand(0, $size)];
		}
		return $filename;
	}
	
	/**
	 * Function: valCacheAge
	 *
	 * Prüft das Alter des Caches und erzeigt seine Verarbeitung, sofern erforderlich
	 * Die Nutzung von DO_EVENTS ist veraltet!
	 *
	 * @param bool $nonIOBlocking  Wurde die verarbeitung von GEARMAN_WORKER_NON_BLOCKING getriggert oder von einem EVENT vom SAT-System
	 * @return void
	 * @since 1.4.3
	 * @version 1.4.4
	 *
	 */
	public static function valCacheAge($nonIOBlocking = false){
		if(self::$last_change < (time() - Config::val('NAGIOS', 'max_wait')) && self::$export_running == false && self::$nagios_runtime_ok === true){
			if(sizeof(self::$cache) > 0){
				if(!$nonIOBlocking){
					Logger::clog('Hand over my cache to Nagios (forced by SAT (DO_EVENTS)) [Cachesize: '.sizeof(self::$cache).' / '.Config::val('NAGIOS', 'max_checks').']', 3);
				}else{
					Logger::clog('Hand over my cache to Nagios (forced by GEARMAN_WORKER_NON_BLOCKING) [Cachesize: '.sizeof(self::$cache).' / '.Config::val('NAGIOS', 'max_checks').']', 3);
				}
				self::processedCache(false, true);
			}
		}
		return true;
	}
	
	/**
	 * Function: _construct
	 *
	 * Ersetzt __construct in der statischen Klasse (static.class.nagios.php || self)
	 * Nur 1 _ ist kein typo, da statische klasse
	 *
	 * @return void
	 * @version 1.7.6
	 * @since 1.4.4
	 *
	 */
	public static function _construct(){
		if(Config::val('NAGIOS', 'nagios4x_compatibility')){
			self::$nagios_runtime_ok = false;
		}else{
			self::$nagios_runtime_ok = true;
		}
		
		if(Config::val('MOD_GEARMAN', 'encryption')){
			self::$MOD_GEARMAN_ENCRYPTION = true;
		}else{
			self::$MOD_GEARMAN_ENCRYPTION = false;
		}
		
		$key = Config::val('MOD_GEARMAN', 'key');
		
		while(strlen($key) < 32){
			$key .= chr(0);
		}
		self::$MOD_GEARMAN_KEY = $key;
	}
	
	/**
	 * Function: toModGearman
	 *
	 * Erzeict einen neuen GearmanClient
	 *
	 * @param  array $ExternalCommandAsArray  Das external Command für Nagios als Array
	 * @param  GearmanClient $gearmanClient  Der GearmanClient zum erstellen des Jobs
	 * @return GearmanClient
	 * @since 1.7.6
	 *
	 */
	public function getModGearmanGearmanClient(){
		$gmclient= new GearmanClient();
		$gmclient->addServer(); //127.0.0.1 4730
		return $gmclient;
	}
	
	/**
	 * Function: toModGearman
	 *
	 * Schreibt die gesammelten Daten in die check_results queue von Mod_Gearman
	 *
	 * @param  array $ExternalCommandAsArray  Das external Command für Nagios als Array
	 * @param  GearmanClient $gearmanClient  Der GearmanClient zum erstellen des Jobs
	 * @return void
	 * @since 1.7.6
	 *
	 */
	public static function toModGearman($ExternalCommandAsArray = array(), $gearmanClient){
		switch($ExternalCommandAsArray['TYPE']){
			case 'SERVICE':
				$data = sprintf(
					"type=%s\nhost_name=%s\nservice_description=%s\nstart_time=%f\nfinish_time=%f\nlatency=%f\nreturn_code=%d\nsource=phpnsta\noutput=%s\n",
					'passive',
					$ExternalCommandAsArray['HOSTNAME'],
					$ExternalCommandAsArray['SERVICEDESC'],
					$ExternalCommandAsArray['start_time'],
					$ExternalCommandAsArray['end_time'],
					$ExternalCommandAsArray['latency'],
					$ExternalCommandAsArray['SERVICESTATEID'],
					$ExternalCommandAsArray['SERVICEOUTPUT'].'|'.$ExternalCommandAsArray['SERVICEPERFDATA']
				);
				break;
			case 'HOST':
				$data = sprintf(
					"type=%s\nhost_name=%s\nstart_time=%f\nfinish_time=%f\nlatency=%f\nreturn_code=%d\nsource=phpnsta\noutput=%s\n",
					'passive',
					$ExternalCommandAsArray['HOSTNAME'],
					$ExternalCommandAsArray['start_time'],
					$ExternalCommandAsArray['end_time'],
					$ExternalCommandAsArray['latency'],
					$ExternalCommandAsArray['HOSTSTATEID'],
					$ExternalCommandAsArray['HOSTOUTPUT']
				);
				break;
		}
		
		if(isset($data)){
			Logger::clog('Data for Mod_Gearman: '.$data, 9);
			if(self::$MOD_GEARMAN_ENCRYPTION === true){
				$gearmanClient->doBackground('check_results', self::encryptForModGearman($data));
			}else{
				$gearmanClient->doBackground('check_results', $data);
			}
		}
	}
	
	/**
	 * Function: encryptForModGearman
	 *
	 * Verschlüsselt einen String für Mod_Gearman
	 *
	 * @param  string $msg  Nachrichtig die Verschlüsselt werden soll
	 * @return string
	 * @since 1.7.6
	 *
	 */
	public static function encryptForModGearman($msg){
		return base64_encode(openssl_encrypt($msg, 'AES-256-ECB', self::$MOD_GEARMAN_KEY, OPENSSL_RAW_DATA));
		//return base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_128, self::$MOD_GEARMAN_KEY, $msg, MCRYPT_MODE_ECB));
	}
}


