<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Statische Klasse, zur Verwaltung der SAT-Systeme
 *
 * Diese Klasse liest die SAT-Systeme aus der openITCOCKPIT Datenbank und erstellt die benötigten $SAT$_nagios.cmd Dateien
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @uses SSH - Statische Klasse welche die Verwaltung der SSH Verbindungen übernimmt
 * @usedby core.php
 * @usedby main.php
 * @usedby SSH
 * @usedby TSYNC
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class SAT{
	
	/** @var array $satellites Alle gefunden SAT-Systeme mit ihrer Konfiguration, als array */
	public static $satellites = array();
	
	/** @var int $count die Anzahl der gefundenen SAT-Systeme */
	public static $count = 0;
	
	/**
	 * Function: fetch
	 *
	 * Findet alle SAT-Systeme die mindestens einen Host und Service angelegt haben.
	 * Das Ergebniss wird als Array in self::$satellites geschrieben und die Anzahl in self::$count 
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function fetch(){
		global $DB;
		$return = array();
		
		if(Config::val('MAIN', 'oitc_version') == 2){
			//openITCOCKPIT V2 find SATs
			$result = $DB->select("SELECT ip, name, COUNT(nag_hosts.id_host) AS 'Counter'
							FROM nag_satellites 
							LEFT JOIN nag_hosts ON nag_satellites.id_satellite = nag_hosts.id_satellite
							WHERE MASTER=0
							AND nag_hosts.host_object_id > 0
							GROUP BY nag_satellites.ip
							HAVING COUNT(nag_hosts.id_host)>0");
						
			if(is_array($result) && !empty($result)){
				self::$count = sizeof($result);
				foreach($result as $SATAsArray){
					$return[] = array(
						'ip' => $SATAsArray['ip'],
						'name' => $SATAsArray['name'],
					);
				}
			}
		}else{
			//openITCOCKPIT V3 find SATs
			$result = $DB->select("SELECT satellites.address, satellites.name, COUNT(hosts.id) AS 'Counter'
							FROM satellites 
							LEFT JOIN hosts ON satellites.id = hosts.satellite_id
							WHERE 1=1
							GROUP BY satellites.address
							HAVING COUNT(hosts.id)>0");
						
			if(is_array($result) && !empty($result)){
				self::$count = sizeof($result);
				foreach($result as $SATAsArray){
					$return[] = array(
						'ip' => $SATAsArray['address'],
						'name' => $SATAsArray['name'],
					);
				}
			}
		}
		
		self::$satellites = $return;
		unset($return);
		unset($result);
	}
	
	/**
	 * Function: createCMDFile
	 *
	 * Erstellt für jedes Satelliten System die $SATNAME$_nagios.cmd
	 * Externe Kommandos, welche über die openITCOCKPIT Oberfläche auf dem Master-System
	 * abgesetzt werden, werden in diese Dateien geschrieben.
	 * Der phpNSTA liest den Inhalt aus und überträgt das Kommando auf den entsprechenden SAT (SAT-COM)
	 *
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function createCMDFile(){
		if(!is_array(self::$satellites) || empty(self::$satellites)){
			self::fetch();
		}
		
		if(is_array(self::$satellites) && !empty(self::$satellites)){
			foreach(self::$satellites as $satellite){
				
				if(Config::val('MAIN', 'oitc_version') == 3){
					$satellite['name'] = md5($satellite['name']);
				}
				
				if(!file_exists(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'))){
					Logger::log('Create file '.Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'), 2);
					touch(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'));
					self::fixPermissions(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'));
				}
			}
		}
	}
	
	/**
	 * Function: fixPermissions
	 *
	 * Setzt für die Übergebene Datei die benötigten Dateirechte, damit der Webserver die Datei beschreiben kann
	 *
	 * @param  string $filename  Kompletter Pfard zur Datei, für die die Dateiberechtigungen gesetzt werden sollen
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function fixPermissions($filename){
		if(file_exists($filename)){
			chown($filename, Config::val('NAGIOS', 'apache_user'));
			chgrp($filename, Config::val('NAGIOS', 'nagios_group'));
			chmod($filename, Config::val('SAT', 'chmod'));
		}
	}
	
	/**
	 * Function: checkCMDFiles
	 *
	 * Prüft die localen $SAT$_nagios.cmd Dateien, ob eine Aufgabe auf ein SAT-System übertragen werden muss
	 *
	 * @return array Ein Array mit allen Aufgaben aus den SAT-System CMD Files
	 * @since 1.0
	 * @version 1.4
	 *
	 */
	public static function checkCMDFiles(){
		if(is_array(self::$satellites) && !empty(self::$satellites)){
			clearstatcache();
			$return = array();
			foreach(self::$satellites as $satellite){
				if(Config::val('SSH', 'use_ssh_tunnel') === true){
					$local_connection = SSH::getLocalIPAndPort($satellite['ip'], $satellite['name']);
				}
				
				if(Config::val('MAIN', 'oitc_version') == 3){
					$satellite['name'] = md5($satellite['name']);
				}
				
				if(file_exists(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'))){
					if(filesize(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix')) > 0 ){
						$return[$satellite['name']] = array();
						if(Config::val('SSH', 'use_ssh_tunnel') === true){
							Logger::clog('SAT-COM: Sending commands to SAT '.$satellite['name'].' ['.$local_connection['local_ip'].':'.$local_connection['local_port'].']', 3);
						}else{
							Logger::clog('SAT-COM: Sending commands to SAT '.$satellite['name'].' ['.$satellite['ip'].']', 3);
						}
						$commands = file(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'));
						if(Config::val('SSH', 'use_ssh_tunnel') === true){
							$return[$satellite['name']]['local_ip'] = $local_connection['local_ip'];
							$return[$satellite['name']]['local_port'] = $local_connection['local_port'];
						}else{
							$return[$satellite['name']]['ip'] = $satellite['ip'];
						}
						$return[$satellite['name']]['commands'] = $commands;	
						unset($commands);
						self::wipe(Config::val('SAT', 'master_cmd_path').$satellite['name'].Config::val('SAT', 'suffix'));
					}
				}
				if(isset($local_connection)){
					unset($local_connection);
				}
			}
		}
		return $return;
	}
	
	/**
	 * Function: wipe
	 *
	 * Kürzt die nagios.cmd eines SAT-Systems auf eine Länge von 0 und setzt die benötigten Dateirechte
	 *
	 * @param  string $filename  Der komplette Dateiname mit Pfard zur Datei
	 * @return void
	 * @since 1.0
	 *
	 */
	public static function wipe($filename){
		fclose(fopen($filename, 'w+'));
		self::fixPermissions($filename);
	}
	
	/**
	 * Function: buildGearmanServerString
	 *
	 * Erstellt für jedes Child den String für die Gearman Function addServers()
	 *
	 * @return Array
	 * @since 1.0
	 * @version 1.4
	 *
	 */
	public static function buildGearmanServerString(){
		$return = array();
		$use_ssh = Config::val('SSH', 'use_ssh_tunnel');
		
		if(!is_array(self::$satellites) || empty(self::$satellites)){
			self::fetch();
		}
		
		if(is_array(self::$satellites) && !empty(self::$satellites)){
			if(Config::val('CPU', 'multithreading')){
				//Die einzelnen Aufgeben (SAT-Systeme) soll auf mehrere Worker verteilt werden
				if(self::$count > Config::val('CPU', 'max_threads')){
					//Es exisiteren mehr SAT-Systeme als CPU Threads erstellt werden sollen. Aus diesem Grund müssen die Worker mehrere SAT-Systeme gleichzeitig abholen
					Logger::log('Notice: You have more satellite systems as allowd CPU threads', 1);
					$tmp = array();
					$i = 0;
					$return = array();
					$SatellitePerWorker = ceil(self::$count / Config::val('CPU', 'max_threads'));
					$return['number_of_workers'] = Config::val('CPU', 'max_threads');
					
					$overflow = self::$count % Config::val('CPU', 'max_threads'); //Rest muss einzeln laufen
					
					
					$tmp_array_satellites = self::$satellites; 
					
					
					if($overflow > 0){
						foreach($tmp_array_satellites as $key =>$tmp_array_satellite){
							if($overflow > 0){
								if($use_ssh === true){
									//Benutze einen SSH-Tunnel um eine Verbindung zu den SAT-Systemen aufzubauen
									$SSHConnectionInfo = SSH::addSatellite($tmp_array_satellite['ip'], $tmp_array_satellite['name']);
									$return[$i]['servers'][] = $SSHConnectionInfo['local_ip'].':'.$SSHConnectionInfo['local_port'];
									unset($SSHConnectionInfo);
								}else{
									//Verwende den standard Web über TCP + Gearman für die Verbindung
									$return[$i]['servers'][] = $tmp_array_satellite['ip'].':'.Config::val('GEARMAN', 'port');
								}
								$overflow--;	
								$i++;
								unset($tmp_array_satellites[$key]);
								continue;
							}
							break;
						}
						$SatellitePerWorker = ceil(sizeof($tmp_array_satellites) / Config::val('CPU', 'max_threads'));
					}
						
					$i=0;
					foreach(array_chunk($tmp_array_satellites, $SatellitePerWorker, true) as $SatellitesAsArray){
						foreach($SatellitesAsArray as $satellite){
							if($use_ssh === true){
								//Benutze einen SSH-Tunnel um eine Verbindung zu den SAT-Systemen aufzubauen
								$SSHConnectionInfo = SSH::addSatellite($satellite['ip'], $satellite['name']);
								$return[$i]['servers'][] = $SSHConnectionInfo['local_ip'].':'.$SSHConnectionInfo['local_port'];
								unset($SSHConnectionInfo);
							}else{
								//Verwende den standard Web über TCP + Gearman für die Verbindung
								$return[$i]['servers'][] = $satellite['ip'].':'.Config::val('GEARMAN', 'port');
							}
							$return[$i]['names'][] = $satellite['name'];
						}
						$return[$i]['server_string'] = implode(',', $return[$i]['servers']);
						$return[$i]['server_names'] = implode(',', $return[$i]['names']);
						if(is_array($return[$i]['names'])){
							$return[$i]['namesAsArray'] = $return[$i]['names'];
						}else{
							$return[$i]['namesAsArray'] = array($return[$i]['names']);
						}
						$i++;
					}
					unset($tmp_array_satellites);
					return $return;
				}else{
					//Für jedes SAT-System kann ein CPU-Thread erstellt werden
					$return = array();
					$i = 0;
					$return['number_of_workers'] = self::$count;
					foreach(self::$satellites as $satellite){
						$return[$i] = array();
						if($use_ssh === true){
							//Benutze einen SSH-Tunnel um eine Verbindung zu den SAT-Systemen aufzubauen
							$SSHConnectionInfo = SSH::addSatellite($satellite['ip'], $satellite['name']);
							$return[$i]['server_string'] = $SSHConnectionInfo['local_ip'].':'.$SSHConnectionInfo['local_port'];
							unset($SSHConnectionInfo);
						}else{
							//Verwende den standard Web über TCP + Gearman für die Verbindung
							$return[$i]['server_string'] = $satellite['ip'].':'.Config::val('GEARMAN', 'port');
						}
						$return[$i]['server_names'] = $satellite['name'];
						$return[$i]['namesAsArray'] = array($satellite['name']);
						$i++;
					}
					return $return;
				}
				
			}else{
				//Alle Aufgaben werden von einem einzigen worker ausgeführt
				$return[0] = array();
				$return['number_of_workers'] = 1;
				$tmp = array();
				$names = array();
				foreach(self::$satellites as $satellite){
					if($use_ssh === true){
						//Benutze einen SSH-Tunnel um eine Verbindung zu den SAT-Systemen aufzubauen
						$SSHConnectionInfo = SSH::addSatellite($satellite['ip'], $satellite['name']);
						$tmp[] = $SSHConnectionInfo['local_ip'].':'.$SSHConnectionInfo['local_port'];
						unset($SSHConnectionInfo);	
					}else{
						//Verwende den standard Web über TCP + Gearman für die Verbindung
						$tmp[] = $satellite['ip'].':'.Config::val('GEARMAN', 'port');	
					}
					$tmp[] = $satellite['name'];
				}
				$return[0]['server_string'] = implode(',', $tmp);
				$return[0]['server_names'] = implode(',', $names);
				$return[0]['namesAsArray'] = $names;
				return $return;
			}
		}
	}
}
