<?php
/**
 * phpNSTA
 *
 * This file is part of the phpNSTA
 *
 * @license http://www.gnu.de/documents/gpl-2.0.de.html
 * @copyright 2013 - 2016
 */

/**
 * Statische Klasse welche die Verwaltung der SSH Verbindungen übernimmt
 *
 * Diese Klasse übernimmt die Verwaltung der Ports, auf die die Verbindung über SSH getunnelt wird
 *
 * @uses Config - Statische Klasse um die Konfigurationsdatei zu lesen
 * @uses Logger - Statische Klasse um in der phpNSTA.log zu schreiben
 * @uses SAT - Statische Klasse, zur Verwaltung der SAT-Systeme
 * @usedby core.php
 * @usedby main.php
 * @usedby API
 * @usedby SSH
 * @usedby Supervisor
 * @usedby SAT
 * @usedby TSYNC
 * @package phpNSTA
 * @author Daniel Ziegler <daniel.ziegler@it-novum.com>
 */
class SSH{
	/** @var array $sat_systems Enthält die Konfiguration (IP, Port, local Port etc...) aller SAT-Systeme als array */
	public static $sat_systems = array();
	
	/** @var array $$used_ports Enthält alle Ports, die vom phpNSTA für den SSH tunnel gerade verwendet werden */
	public static $used_ports = array();
	
	/** @var int $count wird zum prüfen verwendet, ob noch freie Ports in der gewählten Portrange zur Verfügung stehen */
	public static $count = 0;
	
	/**
	 * Function: getPortrange
	 *
	 * Liefert die Portrange für die SSH-Verbindungen als Array zurück
	 *
	 * @return array   Array mit der Portrange
	 * @since 1.4
	 *
	 */
	public static function getPortrange(){
		$PortRangeAsProtoArray = explode('-', Config::val('SSH', 'port_range'));
		if(isset($PortRangeAsProtoArray[0])		 &&
		   is_numeric($PortRangeAsProtoArray[0]) &&
		   isset($PortRangeAsProtoArray[1])		 &&
		   is_numeric($PortRangeAsProtoArray[1])){
			   //Portrange ist gesetzt.
			   return array('start' => $PortRangeAsProtoArray[0], 'end' => $PortRangeAsProtoArray[1], 'available_ports' => ($PortRangeAsProtoArray[1] - $PortRangeAsProtoArray[0]));
		   }else{
			   //Portrange ist nicht gesetzt oder konnte nicht geparst werden
			   return array('start' => 0, 'end' => 0, 'available_ports' => 0);
		   }
	}
	
	/**
	 * Function: getFreePort
	 *
	 * Liefert einen freien Port aus der Portrange für die SSH-Verbindungen zurück
	 *
	 * @return int Freier Port als Integer
	 * @since 1.4
	 *
	 */
	public static function getFreePort(){
		$PortRange = self::getPortrange();
		if($PortRange['available_ports'] > self::$count){
			while(true){
				Logger::log("Trying to find a free port for a new SSH tunnel", 4);
				$new_port = rand($PortRange['start'], $PortRange['end']);
				if(!in_array($new_port, self::$used_ports)){
					//Der generierte Port ist noch frei und kann für eine SSH-Verbindung genutzt werden
					if(self::checkPort(Config::val('SSH', 'local_bind_address'), $new_port)){
						//Der Port ist auf dem lokalen System verfügbar und kann für einen SSH-Tunnel genutzt werden
						Logger::log("Generated new port for SSH tunnel (".$new_port.")", 4);
						return $new_port;
						break;
					}
				}
			}
		}else{
			Logger::log("No more free ports for a new SSH tunnel available!");
		}
	}
	
	/**
	 * Function: checkPort
	 *
	 * Prüft ob der angefragte Port auf SSH => local_bind_address belegt/benutzt wird.
	 *
	 * @param  string $server  Die IP-Adresse des Servers, auf dem der Port geprüft werden soll.
	 * @param  int $port       Die Nummer des Ports welcher geprüft werden soll
	 * @param  string $return  Wie der Rückgabewert von fsockopen ausgewertet werden soll (Default: default)
	 * @return bool            True wenn Port unbeliegt ist, false, wenn der Port schon benutzt wird.
	 * @since 1.4
	 *
	 */
	public static function checkPort($server, $port, $return="default"){
		Logger::log("Checking port: ".$port." on server: ".$server);
		$returncode = @fsockopen($server, $port, $errno, $errstr, Config::val('SSH', 'timeout'));
		if($returncode !== false){
			fclose($returncode);
		}
		switch($return){
			case "is_alive":
				//Wird genutzt um zu prüfen, ob der SSH-Tunnel noch lebt oder getrennt wurde...
				return $returncode;
				break;
				
			default:
				//default wird immer genutzt, wenn geprüft werden soll ob auf 127.0.0.1 der Port frei ist, um einen SSH-Tunnel darauf zu binden.
				//Aus diesem Grund muss der Rückgabewert negiert werden (aus true wird false und aus false wird true)
				if($errstr == "Connection refused"){
					return true;
				}
				return false;
				break;
		}
	}
	
	/**
	 * Function: addSatellite
	 *
	 * Fügt die IP-Adresse eines Sattelitten Systems hinzu, für das Später ein SSH-Tunnel erzeugt werden soll
	 *
	 * @param  string $ip    Die IP-Adresse des SAT-Systems
	 * @param  string $name  Name des SAT-Systems
	 * @return array  Array mit dem gerade hinzugefügten SAT-System
	 * @since 1.4
	 *
	 */
	public static function addSatellite($ip, $name){
		$return = array(
			'name' => $name,
			'ip' => $ip,
			'local_ip' => Config::val('SSH', 'local_bind_address'),
			'local_port' => self::getFreePort(),
			'is_established' => false,
			'resource' => null
		);
		self::$sat_systems[] = $return;
		self::addUsedPort($return['local_port']);
		return $return;
	}
	
	/**
	 * Function: addUsedPort
	 *
	 * Fügt einen neuen Port zur Variable self::$used_ports hinzu.
	 *
	 * @param  int $port  Die Nummer des benutzten Ports
	 * @return void
	 * @since 1.4
	 *
	 */
	public static function addUsedPort($port){
		array_push(self::$used_ports, $port);
	}
	
	/**
	 * Function: checkConnection
	 *
	 * Prüft ob zu dem SAT-System eine Verbindung aufgebaut werden kann
	 *
	 * @param  string $ip    Die IP-Adresse des zu prüfenden SAT-Systems
	 * @param  string $name  Der Name des SAT-Systems
	 * @return void
	 * @since 1.4
	 *
	 */
	public static function checkConnection($ip, $name){
		if(self::checkPort($ip, Config::val('SSH', 'port'), 'is_alive')){
			$connection = ssh2_connect($ip, Config::val('SSH', 'port'));
			if(ssh2_auth_pubkey_file($connection, Config::val('SSH', 'username'), Config::val('SSH', 'public_path'), Config::val('SSH', 'private_path'))){
				return true;
			}else{
				Logger::log("SSH authentication failure on ".$name.'('.$ip.')', 0);
				return false;
			}
			
		}
		Logger::log('Cloud not connect to '.$name.'('.$ip.')', 0);
		return false;
	}
	
	/**
	 * Function: checkAllConnections
	 *
	 * Prüft ob zu jedem SAT-System eine SSH Verbindung aufgebaut werden kann
	 *
	 * @return void
	 * @since 1.4
	 *
	 */
	public static function checkAllConnections(){
		SAT::fetch();
		foreach(SAT::$satellites as $satellite){
			$return = self::checkConnection($satellite['ip'], $satellite['name']);
		}
	}
	
	/**
	 * Function: getLocalIPAndPort
	 *
	 * Gibt die locale IP-Adresse eines SAT-Systems zurück, welches über SSH getunnelt wird.
	 *
	 * @param  string $remote_ip  Die echte IP-Adresse des SAT-Systems
	 * @param  string $name  Der Name des SAT-Systems
	 * @return array  Array mit der localen IP-Adresse und dem localen Port
	 * @since 1.4
	 *
	 */
	public static function getLocalIPAndPort($remote_ip, $name){
		foreach(self::$sat_systems as $sattelite){
			if(in_array($remote_ip, $sattelite) && in_array($name, $sattelite)){
				return array('local_ip' => $sattelite['local_ip'], 'local_port' => $sattelite['local_port']);
			}
		}
	}
}
